#include <stdio.h>
#include <stdlib.h>
#include <sndfile.h>
#include <math.h>
#include <time.h>
#include <omp.h>

#include    "sound.h"
#include    "basics.h"
#include    "options.h"
#include    "functions.h"

int main(int argc, char **argv)
{

/* DECLARATION OF THE VARIABLES */

/* Inputs */
const char
    * inSound = NULL, /*Path to the input audio file*/
    * outSound=NULL, /*Path to the output restored audio file*/
    * outASCIIRestored=NULL,
    * outASCIIBursts;

char outASCIIRestoredBurst[1000];

double K=0; /* First threshold*/
int p=0, /* Order of the model*/
    b=0, /* Second threshold*/
    nWindow=0, /*Window length*/
    nBurstsPlot=0;


/*Internal variables*/
sOptions *O=NULL;
int    i=0,
    j=0,
    k=0,
    nBursts=0,
    maxLength=0,
    minLength=0;

double 	nSamples=0,
	meanLength=0;

Sound 	* S_in=NULL,
        * S_out=NULL;

int * burst1=NULL,
    * burst2=NULL,
    * start_burst=NULL,
    * end_burst=NULL,
    * length_burst=NULL,
    * plot_burst=NULL,
    * select_burst=NULL,
    * start_fig=NULL,
    * end_fig=NULL;

double * min_burst=NULL,
       * max_burst=NULL,
       * maxabs_burst=NULL,
       * maxl1norm_burst=NULL;

FILE *f1=NULL,
     *f2=NULL;


/* READING AND SETTING THE INPUT PARAMETERS */

/* Read the input parameters */
  inSound = argc > 1 ? argv[1] : NULL;
  if (inSound == NULL) return -1;
  O = newOptions(argv + 1, argc-1);

/* Default values for the input parameters */

K    = getDoubleOption(O, "K",   2.0);
b    = getIntOption(O, "b",   20);
p    = getIntOption(O, "p",   3*100+2);
nWindow    = getIntOption(O, "nWindow",   8*p);
nBurstsPlot    = getIntOption(O, "nBurstsPlot",   20);


outSound = getStringOption(O, "outSound", NULL);
outASCIIRestored = getStringOption(O, "outASCIIRestored", NULL);
outASCIIBursts = getStringOption(O, "outASCIIBursts", NULL);



/* MAIN FUNCTION */
srand (time (NULL));

/* Reading the input sound signal*/
S_in=audioRead(inSound,1,20000000);
S_out=audioRead(inSound,1,20000000);




printf("1 %ld %.2f %d %d %d ",S_in->nSamples,K,b,p,nWindow);


burst1=zeros_ivect(S_out->nSamples);
reconstruct_signal(S_out,p,K,b,nWindow,burst1);
analyze_burst(burst1,S_out->nSamples,&start_burst,&end_burst,&length_burst,&nBursts,&nSamples,&minLength,&maxLength,&meanLength);
printf("%d %.2f %d %d %.2f ",nBursts,nSamples,minLength,maxLength,meanLength);

burst2=zeros_ivect(S_out->nSamples);
reconstruct_signal(S_out,p,K,b,nWindow,burst2);
analyze_burst(burst2,S_out->nSamples,&start_burst,&end_burst,&length_burst,&nBursts,&nSamples,&minLength,&maxLength,&meanLength);
printf("%d %.2f %d %d %.2f ",nBursts,nSamples,minLength,maxLength,meanLength);

for (i=0;i<S_out->nSamples;i++) {
	if (burst2[i]==1) {
	burst1[i]=1;
	}
}

analyze_burst(burst1,S_out->nSamples,&start_burst,&end_burst,&length_burst,&nBursts,&nSamples,&minLength,&maxLength,&meanLength);
printf("%d %.2f %d %d %.2f ",nBursts,nSamples,minLength,maxLength,meanLength);

if (outSound)
    audioWrite(S_out,outSound);

if (outASCIIRestored) {


	maxabs_burst=zeros_dvect(nBursts);
	maxl1norm_burst=zeros_dvect(nBursts);

	for (j=0;j<nBursts;j++) {
	maxabs_burst[j]=maxabs_dvect(S_in->channel[0],start_burst[j],end_burst[j],S_in->nSamples,S_out->channel[0],start_burst[j],end_burst[j],S_out->nSamples);
	maxl1norm_burst[j]=maxl1norm_dvect(S_in->channel[0],start_burst[j],end_burst[j],S_in->nSamples,S_out->channel[0],start_burst[j],end_burst[j],S_out->nSamples);
	}


	k=0;
	for (j=0;j<nBursts;j++) {
	    if ((length_burst[j]>4)&(maxl1norm_burst[j]>0.5*maxabs_burst[j])) {
	    k++;
		}
	}
	
	select_burst=zeros_ivect(k);
	k=0;
	for (j=0;j<nBursts;j++) {
	     if ((length_burst[j]>4)&(maxl1norm_burst[j]>0.5*maxabs_burst[j])) {
	    select_burst[k]=j;
	    k++;
            }
	}
	
	if (nBurstsPlot>k)
	nBurstsPlot=k;

	printf("%d\n",nBurstsPlot);	

	rand_unique(0,k-1,nBurstsPlot,&plot_burst);
	for (j=0;j<nBurstsPlot;j++) {
            plot_burst[j]=select_burst[plot_burst[j]];
	}	
	isort(plot_burst, nBurstsPlot);
	
	start_fig=zeros_ivect(nBurstsPlot);
	end_fig=zeros_ivect(nBurstsPlot);

	for (j=0;j<nBurstsPlot;j++) {
	sprintf(outASCIIRestoredBurst,"%s_%d.asc",outASCIIRestored,j);
	f1 = fopen(outASCIIRestoredBurst,"w");
	k=end_burst[plot_burst[j]]-start_burst[plot_burst[j]]+1;
	start_fig[j]=start_burst[plot_burst[j]]-k;
	end_fig[j]=end_burst[plot_burst[j]]+k;
	if ((plot_burst[j]>0)&&(end_burst[plot_burst[j]-1]>start_fig[j]))
		start_fig[j]=end_burst[plot_burst[j]-1];

        if ((plot_burst[j]<nBursts-1)&&(start_burst[plot_burst[j]+1]<end_fig[j]))
		end_fig[j]=start_burst[plot_burst[j]+1];

	
		for (i=start_fig[j];i<end_fig[j]; i++) {
			if ((i>=start_burst[plot_burst[j]])&(i<=end_burst[plot_burst[j]])){
    		fprintf(f1, "%6d %6d %12.7g %12.7g\n", i+1-start_fig[j],i+1-start_fig[j],S_out->channel[0][i],S_in->channel[0][i]); }
			else {
		fprintf(f1, "%6d %6d %12.7g %12.7g\n", i+1-start_fig[j],i+1-start_fig[j],S_in->channel[0][i],S_in->channel[0][i]);}
    	}
    	fclose(f1);
	}



	min_burst=zeros_dvect(nBurstsPlot);
	max_burst=zeros_dvect(nBurstsPlot);
	for (i=0;i<nBurstsPlot;i++){
	    
		for (j=start_fig[i];j<end_fig[i]; j++) {
			if (floor(S_out->channel[0][j]*100)/100<min_burst[i])
			min_burst[i]=floor(S_out->channel[0][j]*100)/100;

			if (floor(S_in->channel[0][j]*100)/100<min_burst[i])
			min_burst[i]=floor(S_in->channel[0][j]*100)/100;

			if (ceil(S_out->channel[0][j]*100)/100>max_burst[i])
			max_burst[i]=ceil(S_out->channel[0][j]*100)/100;

			if (ceil(S_in->channel[0][j]*100)/100>max_burst[i])
			max_burst[i]=ceil(S_in->channel[0][j]*100)/100;
		}
	}


    	f2 = fopen(outASCIIBursts, "w");
    	for (i=0; i<nBurstsPlot; i++) {
		k=end_burst[plot_burst[i]]-start_burst[plot_burst[i]]+1;
    		fprintf(f2, "%6d %6d %.2f %.2f %6d %6d\n",1,end_fig[i]-start_fig[i],min_burst[i],max_burst[i],plot_burst[i]+1,k);
		}
    	fclose(f2);

}






soundClean(S_in);
soundClean(S_out);
  free(burst1);
free(burst2);
   free(start_burst);
   free(max_burst);
   free(min_burst);
   free(end_burst);
free(select_burst);
free(length_burst);
free(maxabs_burst);
free(maxl1norm_burst);
deleteOptions(O);

return 0;

}
